/**
 * Implementar o page up and down
 * botao reset e done
 * traducoes
 */
if (!lemonade && typeof (require) === 'function') {
    var lemonade = require('lemonadejs');
}

if (!Modal && typeof (require) === 'function') {
    var Modal = require('@lemonadejs/modal');
}

; (function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
    typeof define === 'function' && define.amd ? define(factory) :
    global.Dropdown = factory();
}(this, (function () {

    // Default row height
    let defaultRowHeight = 24;

    /**
     * Compare two arrays to see if contains exact the same elements
     * @param {number|number[]} a1
     * @param {number|number[]} a2
     */
    const compareValues = function (a1, a2) {
        if (!a1 || !a2) {
            return false;
        }
        if (!Array.isArray(a1) || !Array.isArray(a2)) {
            if (a1 === a2) {
                return true;
            } else {
                return false;
            }
        } else {
            let i = a1.length;
            if (i !== a2.length) {
                return false;
            }
            while (i--) {
                if (a1[i] !== a2[i]) {
                    return false;
                }
            }
        }
        return true;
    }

    const lazyLoading = function (self) {
        /**
         * Get the position from top of a row by its index
         * @param item
         * @returns {number}
         */
        const getRowPosition = function (item) {
            // Position from top
            let top = 0;
            if (item) {
                let items = self.rows;
                if (items && items.length) {
                    let index = self.rows.indexOf(item);
                    // Go through the items
                    for (let j = 0; j < index; j++) {
                        top += items[j].height || defaultRowHeight;
                    }
                }
            }
            return top;
        }

        const updateScroll = function () {
            let items = self.rows;
            if (items) {
                // Before control
                let before = true;
                // Total of items in the container
                let numOfItems = items.length;
                // Position from top
                let height = 0;
                // Size of the adjustment
                let size = 0;
                // Go through the items
                for (let j = 0; j < numOfItems; j++) {
                    let h = items[j].height || defaultRowHeight;
                    // Height
                    height += h;
                    // Start tracking all items as before
                    if (items[j] === self.result[0]) {
                        before = false;
                    }
                    // Adjustment
                    if (before) {
                        size += h;
                    }
                }
                // Update height
                scroll.style.height = height + 'px';
                // Adjust scroll position
                return size;
            }
            return false;
        }

        const getVisibleRows = function (reset) {
            let items = self.rows;
            if (items) {
                let adjust;
                // Total of items in the container
                let numOfItems = items.length;
                // Get the position from top
                let y = el.scrollTop;
                // Get the height
                let h = null;
                if (self.type === 'searchbar' || self.type === 'picker') {
                    // Priority should be the size used on the viewport
                    h = y + (el.offsetHeight || self.height);
                } else {
                    // Priority is the height define during initialization
                    h = y + (self.height || el.offsetHeight);
                }
                // Go through the items
                let rows = [];
                // Height
                let height = 0;
                // Go through all items
                for (let j = 0; j < numOfItems; j++) {
                    if (items[j].visible !== false) {
                        // Height
                        let rowHeight = items[j].height || defaultRowHeight;
                        // Return on partial width
                        if (height + rowHeight > y && height < h) {
                            rows.push(items[j]);
                        }
                        height += rowHeight;
                    }
                }

                // Update visible rows
                if (reset || !compareValues(rows, self.result)) {
                    // Render the items
                    self.result = rows;
                    // Adjust scroll height
                    let adjustScroll = reset;
                    // Adjust scrolling
                    for (let i = 0; i < rows.length; i++) {
                        // Item
                        let item = rows[i];
                        // Item height
                        let h = item.el.offsetHeight;
                        // Update row height
                        if (!item.height || h !== item.height) {
                            // Keep item height
                            item.height = h;
                            // Adjust total height
                            adjustScroll = true;
                        }
                    }

                    // Update scroll if the height of one element has been changed
                    if (adjustScroll) {
                        // Adjust the scroll height
                        adjust = updateScroll();
                    }
                }

                // Adjust position of the first element
                let position = getRowPosition(self.result[0]);
                let diff = position - el.scrollTop;
                if (diff > 0) {
                    diff = 0;
                }
                self.container.style.top = diff + 'px';

                return adjust;
            }
        }

        /**
         * Move the position to the top and re-render based on the scroll
         * @param reset
         */
        const render = function (reset) {
            // Move scroll to the top
            el.scrollTop = 0;
            // Reset scroll
            updateScroll();
            // Append first batch
            getVisibleRows(reset);
        }

        /**
         * Will adjust the items based on the scroll position offset
         */
        self.adjustPosition = function (item) {
            if (item.el) {
                let h = item.el.offsetHeight;
                let calc = item.el.offsetTop + h;
                if (calc > el.offsetHeight) {
                    let size = calc - el.offsetHeight;
                    if (size < h) {
                        size = h;
                    }
                    el.scrollTop -= -1 * size;
                }
            }
        }

        // Controls
        const scrollControls = function () {
            getVisibleRows(false);
        }

        // Element for scrolling
        let el = self.container.parentNode;
        el.classList.add('lm-lazy');
        // Div to represent the height of the content
        const scroll = document.createElement('div');
        scroll.classList.add('lm-lazy-scroll');
        // Force the height and add scrolling
        el.appendChild(scroll);
        el.addEventListener('scroll', scrollControls, { passive: true });
        el.addEventListener('wheel', scrollControls, { passive: true });
        self.container.classList.add('lm-lazy-items');

        self.goto = function (item) {
            el.scrollTop = getRowPosition(item);
            let adjust = getVisibleRows(false);
            if (adjust) {
                el.scrollTop = adjust;
                // Last adjust on the visible rows
                getVisibleRows(false);
            }
        }

        return (prop) => {
            if (prop === 'rows') {
                render(true);
            }
        }
    }

    const getAttributeName = function(prop) {
        if (prop.substring(0,1) === ':') {
            prop = prop.substring(1);
        } else if (prop.substring(0,3) === 'lm-') {
            prop = prop.substring(3);
        }
        return prop.toLowerCase();
    }

    const extractFromHtml =  function(element) {
        let data = [];
        // Content
        for (let i = 0; i < element.children.length; i++) {
            let e = element.children[i];
            let item = {
                text: e.textContent || e.getAttribute('title'),
                value: e.getAttribute('value'),
            }
            if (item.value == null) {
                item.value = item.text;
            }
            data.push(item);
        }

        return data;
    }

    const extract = function(children) {
        let data = [];

        if (this.tagName) {
            data = extractFromHtml(this);
            // Remove all elements
            this.textContent = '';
        } else {
            // Get data
            if (typeof(children) === 'string') {
                // Version 4
                let d = document.createElement('div');
                d.innerHTML = children;
                data = extractFromHtml(d);
            } else if (children && children.length) {
                // Version 5
                children.forEach((v) => {
                    let item = {}
                    v.props.forEach((prop) => {
                        item[getAttributeName(prop.name)] = prop.value;
                    });
                    if (! item.text) {
                        item.text = v.children[0]?.props[0]?.value || '';
                    }
                    data.push(item);
                });
                // Block children
                children.length = 0;
            }
        }

        return data;
    }

    const Dropdown = function (children) {
        let self = this;
        // Data
        let data = [];
        // Internal value controllers
        let value = [];
        // Cursor
        let cursor = null;
        // Control events
        let ignoreEvents = false;
        // Default width
        if (! self.width) {
            self.width = 260;
        }
        // Lazy loading global instance
        let lazyloading = null;

        // Data
        if (! Array.isArray(self.data)) {
            self.data = [];
        }

        let d = extract.call(this, children);
        if (d) {
            d.forEach((v) => {
                self.data.push(v)
            })
        }

        // Custom events defined by the user
        let onload = self.onload;
        let onchange = self.onchange;

        // Cursor controllers
        const setCursor = function (index, force) {
            let item = self.rows[index];

            if (typeof (item) !== 'undefined') {
                // Set cursor number
                cursor = index;
                // Set visual indication
                item.cursor = true;
                // Go to the item on the scroll in case the item is not on the viewport
                if (!(item.el && item.el.parentNode) || force === true) {
                    // Goto method
                    self.goto(item);
                }
                // Adjust cursor position
                setTimeout(function () {
                    self.adjustPosition(item);
                });
            }
        }

        const removeCursor = function (reset) {
            if (cursor !== null) {
                if (typeof (self.rows[cursor]) !== 'undefined') {
                    self.rows[cursor].cursor = false;
                }
                if (reset) {
                    // Cursor is null
                    cursor = null;
                }
            }
        }

        const moveCursor = function (direction, jump) {
            // Remove cursor
            removeCursor();
            // Last item
            let last = self.rows.length - 1;
            if (jump) {
                if (direction < 0) {
                    cursor = 0;
                } else {
                    cursor = last;
                }
            } else {
                // Position
                if (cursor === null) {
                    cursor = 0;
                } else {
                    // Move previous
                    cursor = cursor + direction;
                }
                // Reach the boundaries
                if (direction < 0) {
                    // Back to the last one
                    if (cursor < 0) {
                        cursor = last;
                    }
                } else {
                    // Back to the first one
                    if (cursor > last) {
                        cursor = 0;
                    }
                }
            }
            // Add cursor
            setCursor(cursor);
        }

        const adjustDimensions = function(data) {
            // Estimate width
            let width = self.width;
            // Adjust the width
            let w = self.input.offsetWidth;
            if (width < w) {
                width = w;
            }
            // Width && values
            data.map(function (s) {
                // Estimated width of the element
                if (s.text) {
                    let w = Math.max(width, s.text.length * 7.5);
                    if (width < w) {
                        width = w;
                    }
                }
            });
            // Min width for the container
            self.container.parentNode.style.width = (width - 2) + 'px';
            // Height
            self.height = 400;

            // Animation for mobile
            if (document.documentElement.clientWidth < 800) {
                self.animation = true;
            }
        }

        const setData = function () {
            // Data
            data = JSON.parse(JSON.stringify(self.data));
            // Re-order to make sure groups are in sequence
            if (data && data.length) {
                // Adjust width and height
                adjustDimensions(data);
                // Groups
                data.sort((a, b) => {
                    // Compare groups
                    if (a.group && b.group) {
                        return a.group.localeCompare(b.group);
                    }
                    return 0;
                });
                let group = '';
                // Define group headers
                data.map((v) => {
                    // Compare groups
                    if (v && v.group && v.group !== group) {
                        v.header = v.group;
                        group = v.group;
                    }
                });
            }
            // Data to be listed
            self.rows = data;
        }

        const updateLabel = function () {
            if (value && value.length) {
                self.input.textContent = value.filter(v => v.selected).map(i => i.text).join('; ');
            } else {
                self.input.textContent = '';
            }
        }

        const setValue = function (v, ignoreEvent) {
            // Values
            let newValue;
            if (!Array.isArray(v)) {
                if (typeof (v) === 'string') {
                    newValue = v.split(';');
                } else {
                    newValue = [v];
                }
            } else {
                newValue = v;
            }

            // Width && values
            value = [];

            if (Array.isArray(data)) {
                data.map(function (s) {
                    // Select values
                    if (newValue.indexOf(s.value) !== -1) {
                        s.selected = true;
                        value.push(s);
                    } else {
                        s.selected = false;
                    }
                });
            }

            // Update label
            updateLabel();

            // Component onchange
            if (! ignoreEvent && typeof(onchange) === 'function') {
                onchange.call(self, self, getValue());
            }
        }

        const getValue = function () {
            if (self.multiple) {
                if (value && value.length) {
                    return value.filter(v => v.selected).map(i => i.value);
                }
                return [];
            } else {
                if (value && value.length) {
                    return value[0].value;
                } else {
                    return '';
                }
            }
        }

        const onclose = function () {
            // Cursor
            removeCursor(true);
            // Reset search
            if (self.autocomplete) {
                // Go to begin of the data
                self.rows = data;
                // Remove editable attribute
                self.input.removeAttribute('contenteditable');
                // Clear input
                self.input.textContent = '';
            }

            // Current value
            let newValue = getValue();

            // If that is different from the component value
            if (!compareValues(newValue, self.value)) {
                self.value = newValue;
            } else {
                // Update label
                updateLabel();
            }

            // Identify the new state of the dropdown
            self.state = false;

            if (typeof (self.onclose) === 'function') {
                self.onclose(self);
            }
        }

        const onopen = function () {
            self.state = true;
            // Value
            let v = value[value.length - 1];
            // Make sure goes back to the top of the scroll
            if (self.container.parentNode.scrollTop > 0) {
                self.container.parentNode.scrollTop = 0;
            }
            // Move to the correct position
            if (v) {
                // Mark the position of the cursor to the same element
                setCursor(self.rows.indexOf(v), true);
            }
            // Prepare search field
            if (self.autocomplete) {
                // Clear input
                self.input.textContent = '';
                // Editable
                self.input.setAttribute('contenteditable', true);
                // Focus on the item
                self.input.focus();
            }

            if (typeof (self.onopen) === 'function') {
                self.onopen(self);
            }
        }

        const loadData = function(result) {
            // Loading controls
            lazyloading = lazyLoading(self);
            // Loading new data from a remote source
            if (result) {
                result.forEach((v) => {
                    self.data.push(v);
                });
            }
            // Process the data
            setData();
            // Set value
            if (typeof(self.value) !== 'undefined') {
                setValue(self.value, true);
            }
            // Custom event by the developer
            if (typeof(onload) === 'function') {
                onload(self);
            }
        }

        self.add = async function (e) {
            if (!self.input.textContent) {
                return false;
            }

            e.preventDefault();

            // New item
            let s = {
                text: self.input.textContent,
                value: self.input.textContent,
            }

            // Event
            if (typeof (self.onbeforeinsert) === 'function') {
                let elClass = self.el.classList;
                elClass.add('lm-dropdown-loading');
                let ret = await self.onbeforeinsert(self, s);
                elClass.remove('lm-dropdown-loading');
                if (ret === false) {
                    return;
                } else if (ret) {
                    s = ret;
                }
            }

            // Process the data
            data.unshift(s);
            // Select the new item
            self.select(e, s);
            // Close dropdown
            self.search();

            // Event
            if (typeof (self.oninsert) === 'function') {
                self.oninsert(self, s);
            }
        }

        self.search = function () {
            if (self.state && self.autocomplete) {
                // Filter options
                let temp;
                let value = self.input.textContent.toLowerCase()
                if (! value) {
                    temp = data;
                } else {
                    temp = data.filter(item => {
                        return item.selected === true ||
                            (item.text.toLowerCase().includes(value)) ||
                            (item.group && item.group.toLowerCase().includes(value)) ||
                            (item.keywords.toLowerCase().includes(value));
                    });
                }
                // Cursor
                removeCursor(true);
                // Update the data from the dropdown
                self.rows = temp;
            }
        }

        self.open = function () {
            if (self.modal && self.modal.closed) {
                // Open the modal
                self.modal.closed = false;
            }
        }

        self.close = function () {
            // Close the modal
            if (self.modal) {
                self.modal.closed = true;
            }
        }

        self.toggle = function () {
            if (self.modal) {
                if (self.modal.closed) {
                    self.open();
                } else {
                    self.close();
                }
            }
        }

        self.click = function (e) {
            if (self.autocomplete) {
                let x;
                if (e.changedTouches && e.changedTouches[0]) {
                    x = e.changedTouches[0].clientX;
                } else {
                    x = e.clientX;
                }
                if (e.target.offsetWidth - (x - e.target.offsetLeft) < 20) {
                    self.toggle();
                } else {
                    self.open();
                }
            } else {
                self.toggle();
            }
        }

        self.select = function (e, s) {
            if (s) {
                if (self.multiple === true) {
                    let position = value.indexOf(s);
                    if (position === -1) {
                        value.push(s);
                        s.selected = true;
                    } else {
                        value.splice(position, 1);
                        s.selected = false;
                    }
                } else {
                    if (value[0] === s) {
                        if (self.allowempty === false) {
                            s.selected = true;
                        } else {
                            s.selected = !s.selected;
                        }
                    } else {
                        if (value[0]) {
                            value[0].selected = false;
                        }
                        s.selected = true;
                    }
                    if (s.selected) {
                        value = [s];
                    } else {
                        value = [];
                    }
                    // Close the modal
                    self.close();
                }
            }
        }

        self.onload = function () {
            if (self.type !== "inline") {
                // Create modal instance
                self.modal = {
                    closed: true,
                    focus: false,
                    onopen: onopen,
                    onclose: onclose,
                    position: 'absolute',
                    'auto-adjust': true,
                    'auto-close': false,
                };
                // Generate modal
                Modal(self.el.children[1], self.modal);
            } else {
                // For inline dropdown
                self.el.setAttribute('tabindex', 0);
                // Remove search
                self.input.remove();
            }

            // Default width
            if (self.width) {
                // Dropdown
                self.el.style.width = self.width + 'px';
            }
            // Adjust width and height
            adjustDimensions(self.data);

            // Focus out of the component
            self.el.addEventListener('focusout', function (e) {
                if (self.modal) {
                    if (!(e.relatedTarget && self.el.contains(e.relatedTarget)) && !self.el.contains(e.relatedTarget)) {
                        if (!self.modal.closed) {
                            self.modal.closed = true;
                        }
                    }
                }
            });

            // Key events
            self.el.addEventListener('keydown', function (e) {
                if (! self.modal.closed) {
                    let prevent = false;
                    if (e.key === 'ArrowUp') {
                        moveCursor(-1);
                        prevent = true;
                    } else if (e.key === 'ArrowDown') {
                        moveCursor(1);
                        prevent = true;
                    } else if (e.key === 'Home') {
                        moveCursor(-1, true);
                        if (!self.autocomplete) {
                            prevent = true;
                        }
                    } else if (e.key === 'End') {
                        moveCursor(1, true);
                        if (!self.autocomplete) {
                            prevent = true;
                        }
                    } else if (e.key === 'Enter') {
                        self.select(e, self.rows[cursor]);
                        prevent = true;
                    } else if (e.key === 'Escape') {
                        self.modal.closed = true;
                        prevent = true;
                    } else {
                        if (e.keyCode === 32 && !self.autocomplete) {
                            self.select(e, self.rows[cursor]);
                        }
                    }

                    if (prevent) {
                        e.preventDefault();
                        e.stopImmediatePropagation();
                    }
                } else {
                    if (e.key === 'ArrowUp' || e.key === 'ArrowDown' || e.key === 'Enter') {
                        self.modal.closed = false;
                    }
                }
            });

            // Load remote data
            if (self.url) {
                fetch(self.url, {
                    headers: {
                        'Content-Type': 'text/json',
                    }
                }).then(r => r.json()).then(loadData);
            } else {
                loadData();
            }
        }

        self.onchange = function (prop) {
            if (prop === 'value') {
                setValue(self.value);
            } else if (prop === 'data') {
                setData();
                self.value = null;
            }

            if (typeof (lazyloading) === 'function') {
                lazyloading(prop);
            }
        }

        /**
         * Sanitize any HTML from be paste on the search
         * @param e
         */
        self.onpaste = function (e) {
            let text;
            if (e.clipboardData || e.originalEvent.clipboardData) {
                text = (e.originalEvent || e).clipboardData.getData('text/plain');
            } else if (window.clipboardData) {
                text = window.clipboardData.getData('Text');
            }
            text = text.replace(/(\r\n|\n|\r)/gm, "");
            document.execCommand('insertText', false, text)
            e.preventDefault();
        }

        return `<div class="lm-dropdown" data-insert="{{self.insert}}" data-type="{{self.type}}" data-state="{{self.state}}" :value="self.value" :data="self.data">
            <div class="lm-dropdown-header">
                <div class="lm-dropdown-input" onpaste="self.onpaste" oninput="self.search" onmousedown="self.click" placeholder="{{self.placeholder}}" :ref="self.input" tabindex="0"></div>
                <div class="lm-dropdown-add" onmousedown="self.add"></div>
                <div class="lm-dropdown-header-controls">
                    <button onclick="self.close" class="lm-dropdown-done">Done</button>
                </div>
            </div>
            <div class="lm-dropdown-content">
                <div>
                    <div :loop="self.result" :ref="self.container" :rows="self.rows">
                        <div class="lm-dropdown-item" onclick="self.parent.select" data-cursor="{{self.cursor}}" data-selected="{{self.selected}}" data-group="{{self.header}}">
                            <div><img :src="self.image" /> <div>{{self.text}}</div></div>
                        </div>
                    </div>
                </div>
            </div>
        </div>`;
    }

    lemonade.setComponents({ Dropdown: Dropdown });

    lemonade.createWebComponent('dropdown', Dropdown);

    return function (root, options) {
        if (typeof (root) === 'object') {
            lemonade.render(Dropdown, root, options)
            return options;
        } else {
            return Dropdown.call(this, root)
        }
    }
})));